@@ -1,6 +1,6 @@
 /*
     showfont:  An example of using the SDL_ttf library with 2D graphics.
-    Copyright (C) 1997, 1998, 1999, 2000, 2001  Sam Lantinga
+    Copyright (C) 1997-2004 Sam Lantinga
 
     This library is free software; you can redistribute it and/or
     modify it under the terms of the GNU Library General Public
@@ -20,12 +20,17 @@
     slouken@libsdl.org
 */
 
-/* $Id: showfont.c,v 1.5 2001/12/17 04:27:35 slouken Exp $ */
+/* $Id: showfont.c,v 1.9 2004/01/04 17:41:55 slouken Exp $ */
 
 /* A simple program to test the text rendering feature of the TTF library */
 
 #include <stdlib.h>
 #include <stdio.h>
+#include <string.h>
+
+#ifdef HAVE_ICONV
+#include <iconv.h>
+#endif
 
 #include "SDL.h"
 #include "SDL_ttf.h"
@@ -35,8 +40,7 @@
 #define NUM_COLORS      256
 
 static char *Usage =
-"Usage: %s [-utf8|-unicode] [-b] [-i] [-u] [-fgcol r,g,b] [-bgcol r,g,b] \
-<font>.ttf [ptsize] [text]\n";
+"Usage: %s [-solid] [-utf8|-unicode] [-b] [-i] [-u] [-fgcol r,g,b] [-bgcol r,g,b] <font>.ttf [ptsize] [text]\n";
 
 int main(int argc, char *argv[])
 {
@@ -244,17 +248,42 @@ int main(int argc, char *argv[])
 
 	    case RENDER_UNICODE:
 		{
-			/* This doesn't actually work because you can't pass
-			   UNICODE text in via command line, AFAIK, but...
-			 */
 			Uint16 unicode_text[BUFSIZ];
 			int index;
-			for ( index = 0; (message[0] || message[1]); ++index ) {
-				unicode_text[index]  = ((Uint8 *)message)[0];
-				unicode_text[index] <<= 8;
-				unicode_text[index] |= ((Uint8 *)message)[1];
-				message += 2;
+#ifdef HAVE_ICONV
+			/* Use iconv to convert the message into utf-16.
+			 * "char" and "" are aliases for the local 8-bit encoding */
+			iconv_t cd;
+			ICONV_CONST char *from_str = message;
+			char *to_str = (char*)unicode_text;
+			size_t from_sz = strlen(message) + 1;
+			size_t to_sz = sizeof(unicode_text);
+			size_t res;
+			int i;
+
+			if ((cd = iconv_open("UTF-16", "char")) == (iconv_t)-1
+			    && (cd = iconv_open("UTF-16", "")) == (iconv_t)-1) {
+				perror("Couldn't open iconv");
+				exit(1);
 			}
+
+			res = iconv(cd, &from_str, &from_sz, &to_str, &to_sz);
+			if (res == -1) {
+				perror("Couldn't use iconv");
+				exit(1);
+			}
+
+			iconv_close(cd);
+#else
+			/* Convert the message from ascii into utf-16.
+			 * This is unreliable as a test because it always
+			 * gives the local ordering. */
+			for (index = 0; message[index]; index++) {
+				unicode_text[index] = message[index];
+			}
+			unicode_text[index] = 0;
+#endif
+
 			if ( rendersolid ) {
 				text = TTF_RenderUNICODE_Solid(font,
 					unicode_text, *forecol);
